'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * text to speech queue handler that uses HTML5 speechSynthesis
 * Voices (en-US):
 * [
 *  "Alex","Agnes","Albert",
 *  "Bad News","Bahh","Bells","Boing","Bruce","Bubbles",
 *  "Cellos",
 *  "Deranged",
 *  "Fred",
 *  "Good News",
 *  "Hysterical",
 *  "Junior",
 *  "Kathy",
 *  "Pipe Organ",
 *  "Princess",
 *  "Ralph",
 *  "Samantha",
 *  "Trinoids",
 *  "Vicki",
 *  "Victoria",
 *  "Whisper",
 *  "Zarvox"
 *  ]
 * @file speak
 * @module speak
 * @author Gregor Adams <greg@pixelass.com>
 */

/**
 * HTML5 speechSynthesis queue handler
 */
var Speak = function () {
  /**
   * create a speaker and synth instance
   * @constructor
   * @param  {Object} global window
   * @return {Function} returns the `speak method`
   */
  function Speak(global) {
    _classCallCheck(this, Speak);

    this.speaker = global.speechSynthesis;
    this.synth = new SpeechSynthesisUtterance();
    this.global = global;
    /**
     * a queue of texts to be spoken when the last
     * one has finished
     * @type {Array}
     */
    this.queue = [];
    /**
     * assign the `onEnd` method to the synths
     * `onend` listener
     * @type {Function}
     */
    this.synth.onend = this.onEnd.bind(this);
    /**
     * return the `speak` method
     */
    return this.speak.bind(this);
  }

  /**
   * a promise that returns a sorted object
   * @return {Promise}
   */


  _createClass(Speak, [{
    key: 'loadVoices',
    value: function loadVoices() {
      var _this = this;

      return new Promise(function (resolve, reject) {
        if (_this.speaker.getVoices().length) {
          resolve(_this.voices);
        }
        try {
          _this.speaker.onvoiceschanged = function (e) {
            _this.updateVoices();
            resolve(_this.voices);
          };
        } catch (err) {
          reject(err);
        }
      });
    }

    /**
     * updates the voices and sorts them
     * @return {Object} sorted voices
     */

  }, {
    key: 'updateVoices',
    value: function updateVoices() {
      var voices = {};
      var raw = this.speaker.getVoices();
      raw.forEach(function (voice) {
        voices[voice.lang] = voices[voice.lang] || {};
        voices[voice.lang][voice.name] = voice;
      });
      this.voices = voices;
    }

    /**
     * returns voices
     * @return {Object} current voices
     */

  }, {
    key: 'onEnd',


    /**
     * handler to manage the queue when a speech has ended.
     */
    value: function onEnd() {
      // allow new speech
      this.speaking = false;
      // check for queue items
      if (this.queue[0]) {
        // items found:
        // play the next item and remove it from the queue
        var next = this.queue[0];
        this.speak(next.message, next.options);
        this.queue.splice(0, 1);
      }
    }

    /**
     * method that actually speaks.
     * This method checks if speaker is already speaking.
     * If `true` the new message and options will be added to
     * the queue, otherwise the message is spoken
     * @param  {String} message - the text that will be synthesised when the
     *                            utterance is spoken.
     * @param  {Object} options - options to use for the utterance.
     * @param  {Number} options.pitch - the pitch at which the utterance will
     *                                  be spoken at.
     * @param  {Number} options.rate - the speed at which the utterance will
     *                                 be spoken at.
     * @param  {Number} options.volume - the volume that the utterance will
     *                                   be spoken at.
     * @param  {String} options.lang - the language of the utterance.
     * @param  {Object} options.voice - the voice that will be used to speak 
     *                                  the utterance.
     */

  }, {
    key: 'speak',
    value: function speak() {
      var _this2 = this;

      var message = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];
      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      if (this.speaking) {
        this.queue.push({ message: message, options: options });
      } else {
        this.speaking = true;
        this.loadVoices().then(function (voices) {
          // define some safe options to make make sure only
          // [lang,pitch,rate,volume, voice] can be changed and have defaults
          var lang = options.lang;
          var voice = options.voice;

          var getVoice = function getVoice() {
            var lang = arguments.length <= 0 || arguments[0] === undefined ? 'en-US' : arguments[0];

            if (lang in voices && voice in voices[lang]) {
              return voices[lang][voice];
            } else {
              return false;
            }
          };
          var safeOptions = {
            lang: lang || 'en-US',
            pitch: options.pitch || 1,
            rate: options.rate || 1,
            volume: options.volume || 1,
            voice: getVoice(lang),
            text: message
          };
          // block next messages (force to queue)
          // apply options and speak
          Object.assign(_this2.synth, safeOptions);
          _this2.speaker.speak(_this2.synth);
        }).catch(function (err) {
          throw err;
        });
      }
    }
  }, {
    key: 'voices',
    get: function get() {
      return this._voices;
    }

    /**
     * sets voices
     * @param  {Object} voices - sorted voices
     */
    ,
    set: function set(voices) {
      this._voices = voices;
    }
  }]);

  return Speak;
}();

exports.default = Speak;